<?php

/**
 * @file
 * Administration functionality for the sOnr webMining Content Aggregator.
 */

/******************************************************
 * Configuration set functions
 *****************************************************/

/**
 * List all saved sOnr webMining configurations.
 *
 * @return string
 *   The rendered HTML of the list of configurations.
 */
function sonr_webmining_configuration_list() {
  $config_sets = sonr_webmining_config_load_multiple();

  $rows = array();
  foreach ($config_sets as $config) {
    $actions = array(
      l(t('Edit'), 'admin/config/services/sonr_webmining/' . $config->getId() . '/edit'),
      l(t('Delete'), 'admin/config/services/sonr_webmining/' . $config->getId() . '/delete'),
    );
    $rows[] = array(
      $config->getTitle(),
      $config->getConnection()->getUrl(),
      '<div class="sonr-webmining-led" data-id="' . $config->getId() . '" title="Checking service"></div>',
      implode(' | ', $actions),
    );
  }
  $table = array(
    'header' => array(
      t('Name'),
      t('URL to sOnr server'),
      t('Available'),
      t('Actions'),
    ),
    'rows' => $rows,
  );

  // Add JavaScript and CSS
  drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js');
  drupal_add_css(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css');

  return theme('table', $table);
}

/**
 * Ajax callback function for checking if a new sOnr server is available.
 */
function sonr_webmining_new_available($form, $form_state) {
  $available = '<div id="health_info" class="available">' . t('The server is available.') . '</div>';
  $not_available = '<div id="health_info" class="not-available">' . t('The server is not available or the credentials are incorrect.') . '</div>';

  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Create a new connection (without saving) with the current form data.
    $connection = SemanticConnector::getConnection('sonr_server');
    $connection->setURL($form_state['values']['url']);
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));
    $sonr_api = $connection->getAPI();
    return $sonr_api->available() ? $available : $not_available;
  }

  return $not_available;
}

/**
 * Ajax callback function for checking if a sOnr server is available.
 */
function sonr_webmining_available($config) {
  $sonr_api = $config->getConnection()->getAPI();
  echo $sonr_api->available() ? 1 : 0;
  exit();
}

/**
 * The form to add or edit a sOnr webMining configuration.
 *
 * @param array $form
 *   The form array.
 *
 * @oaram array &$form_state
 *   The form_state array.
 *
 * @param object $config
 *   A SonrWebminingConfigurationSet-instance
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_configuration_set_form($form, &$form_state, $config = NULL) {
  if (!is_null($config) && !empty($config)) {
    $is_configuration_new = FALSE;
    $form['swid'] = array(
      '#type' => 'hidden',
      '#value' => $config->getId(),
    );
  }
  else {
    $is_configuration_new = TRUE;
    $config = new SonrWebminingConfigurationSet();
  }
  $configuration = $config->getConfig();

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the sOnr webMining configuration.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $config->getTitle(),
    '#required' => TRUE,
  );

  // Define the container for the vertical tabs.
  $form['settings'] = array(
    '#type' => 'vertical_tabs',
  );

  // Tab: Server settings.
  $form['server_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('sOnr server settings'),
    '#group' => 'settings',
  );

  $connections = SemanticConnector::getConnectionsByType('sonr_server');
  if (!empty($connections)) {
    $connection_options = array();
    foreach ($connections as $connection) {
      $credentials = $connection->getCredentials();
      $connection_options[implode('|', array($connection->getTitle(), $connection->getURL(), $credentials['username'], $credentials['password']))] = $connection->getTitle();
    }
    $form['server_settings']['load_connection'] = array(
      '#type' => 'select',
      '#title' => t('Load an available sOnr server'),
      '#options' => $connection_options,
      '#empty_option' => '',
      '#default_value' => '',
    );

    $form['server_settings']['load_connection_button'] = array(
      '#type' => 'button',
      '#value' => t('Load'),
      '#attributes' => array(
        'onclick' => '
        var connection_value = (jQuery(\'#edit-load-connection\').val());
        if (connection_value.length > 0) {
          var connection_details = connection_value.split(\'|\');
          jQuery(\'#edit-server-title\').val(connection_details[0]);
          jQuery(\'#edit-url\').val(connection_details[1]);
          jQuery(\'#edit-name\').val(connection_details[2]);
          jQuery(\'#edit-pass\').val(connection_details[3]);
        }
        return false;',
      ),
    );
  }

  $connection = $config->getConnection();
  $form['server_settings']['connection_id'] = array(
    '#type' => 'hidden',
    '#value' => $connection->getID(),
  );

  $form['server_settings']['server_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Server title'),
    '#description' => t('A short title for the server below.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $connection->getTitle(),
    '#required' => TRUE,
  );

  $form['server_settings']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('URL, where the PoolParty server runs.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $connection->getURL(),
    '#required' => TRUE,
  );

  $credentials = $connection->getCredentials();
  $form['server_settings']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Name of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $credentials['username'],
  );

  $form['server_settings']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#description' => t('Password of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 128,
    '#default_value' => $credentials['password'],
  );

  $form['server_settings']['health_check'] = array(
    '#type' => 'button',
    '#value' => t('Health check'),
    '#ajax' => array(
      'callback' => 'sonr_webmining_new_available',
      'wrapper' => 'health_info',
      'method' => 'replace',
      'effect' => 'slide',
    ),
  );

  if ($is_configuration_new) {
    $markup = '<div id="health_info">' . t('Click to check if the server is available.') . '</div>';
  }
  else {
    $sonr_api = $connection->getAPI();
    $available = '<div id="health_info" class="available">' . t('The server is available.') . '</div>';
    $not_available = '<div id="health_info" class="not-available">' . t('The server is not available or the credentials are incorrect.') . '</div>';
    $markup = $sonr_api->available() ? $available : $not_available;
  }
  $form['server_settings']['health_info'] = array(
    '#markup' => $markup,
  );

  // Tab: Result settings.
  $form['result_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Result settings'),
    '#group' => 'settings',
  );

  $form['result_settings']['items_per_page'] = array(
    '#type' => 'textfield',
    '#title' => t('Items per page'),
    '#description' => t('The number of items you want to display on the first result page.'),
    '#size' => 15,
    '#maxlength' => 5,
    '#default_value' => $configuration['items_per_page'],
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['result_settings']['summary_max_chars'] = array(
    '#type' => 'textfield',
    '#title' => t('Length of the summary'),
    '#description' => t('The maximum length in characters you want to display on every result item.'),
    '#size' => 15,
    '#maxlength' => 5,
    '#default_value' => $configuration['summary_max_chars'],
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['result_settings']['first_page_only'] = array(
    '#type' => 'checkbox',
    '#title' => t('First page only'),
    '#description' => t('This option disables infinite loading as well as any kind of pagination.'),
    '#default_value' => $configuration['first_page_only'],
  );

  $form['result_settings']['show_sentiment'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show dynamic sentiment'),
    '#description' => t('This option specifies whether dynamic sentiments should be displayed or not'),
    '#default_value' => $configuration['show_sentiment'],
  );

  $form['result_settings']['show_tags'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show a list of all tagged concepts for every content item.'),
    '#description' => t('This option specifies whether tags should be displayed or not'),
    '#default_value' => $configuration['show_tags'],
  );

  $form['result_settings']['tags_max_items_container'] = array(
    '#type' => 'container',
    '#attributes' => array('style' => array('padding-left:18px;')),
  );

  $form['result_settings']['tags_max_items_container']['tags_max_items'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of tags'),
    '#description' => t('The maximum number of tags you want to display. Leave empty to display all tags.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['tags_max_items'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="show_tags"]' => array('checked' => TRUE)),
    ),
  );

  $form['result_settings']['show_similar'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show similar documents'),
    '#description' => t('This specifies whether the more like this option should be displayed or not'),
    '#default_value' => $configuration['show_similar'],
  );

  $form['result_settings']['similar_max_items_container'] = array(
    '#type' => 'container',
    '#attributes' => array('style' => array('padding-left:18px;')),
  );

  $form['result_settings']['similar_max_items_container']['similar_max_items'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of documents'),
    '#description' => t('The maximum number of similar documents you want to display.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['similar_max_items'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="show_similar"]' => array('checked' => TRUE)),
      'required' => array(':input[name="show_similar"]' => array('checked' => TRUE)),
    ),
  );

  // Tab: Filter settings.
  $form['filter_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter settings'),
    '#group' => 'settings',
  );

  $form['filter_settings']['separate_blocks'] = array(
    '#type' => 'checkbox',
    '#title' => t('Separate blocks for filters and the result list'),
    '#description' => t('If this option is selected, two blocks will be created. This offers more flexibility in positioning the filters-area or even hiding the filters completely.'),
    '#default_value' => $configuration['separate_blocks'],
  );

  // Update an existing configuration set.
  if ($is_configuration_new) {
    $form['filter_settings']['facets_to_show'] = array(
      '#type' => 'hidden',
      '#value' => $configuration['facets_to_show'],
    );
  }
  else {
    $aggregator = new SonrWebmining($config->getId());
    $all_facets = $aggregator->getAllFacets();
    $stored_facets = array();
    $facets = array();
    foreach ($configuration['facets_to_show'] as $weight => $facet) {
      // Manage new facet data.
      if (is_array($facet)) {
        // Ignore removed facets on sOnr server.
        if (!isset($all_facets[$facet['facet_id']])) {
          continue;
        }
        $facets[$weight] = $facet['facet_id'];
        $stored_facets[$facet['facet_id']] = $facet;
      }
      // Manage old facet data.
      else {
        // Ignore removed facets on sOnr server.
        if (!isset($all_facets[$facet])) {
          continue;
        }
        $facets[$weight] = $facet;
        $stored_facets[$facet] = array(
          'facet_id' => $facet,
          'name' => '',
          'selected' => TRUE,
          'aggregated' => FALSE,
          'max_items' => 10,
        );
      }
    }
    foreach (array_keys($all_facets) as $facet_id) {
      if (!in_array($facet_id, $facets)) {
        $facets[] = $facet_id;
        $stored_facets[$facet_id] = array(
          'facet_id' => $facet_id,
          'name' => $all_facets[$facet_id],
          'selected' => FALSE,
          'aggregated' => FALSE,
          'max_items' => 10,
        );
      }
    }
    foreach ($facets as $weight => $facet_id) {
      $form['filter_settings']['facets_to_show']['#tree'] = TRUE;
      $form['filter_settings']['facets_to_show'][$facet_id]['node'] = array(
        '#markup' => $all_facets[$facet_id],
      );
      // This field is invisible, but contains sort info (weights).
      $form['filter_settings']['facets_to_show'][$facet_id]['weight'] = array(
        '#type' => 'weight',
        // Weights from -255 to +255 are supported because of this delta.
        '#delta' => 255,
        '#title_display' => 'invisible',
        '#default_value' => $weight,
      );
      $form['filter_settings']['facets_to_show'][$facet_id]['name'] = array(
        '#type' => 'textfield',
        '#size' => 15,
        '#maxlength' => 255,
        '#default_value' => isset($stored_facets[$facet_id]['name']) ? $stored_facets[$facet_id]['name'] : $all_facets[$facet_id],
      );
      $form['filter_settings']['facets_to_show'][$facet_id]['selected'] = array(
        '#type' => 'checkbox',
        '#default_value' => $stored_facets[$facet_id]['selected'],
      );
      $form['filter_settings']['facets_to_show'][$facet_id]['aggregated'] = array(
        '#type' => 'checkbox',
        '#default_value' => $stored_facets[$facet_id]['aggregated'],
      );
      $form['filter_settings']['facets_to_show'][$facet_id]['max_items'] = array(
        '#type' => 'textfield',
        '#element_validate' => array('element_validate_integer_positive'),
        '#size' => 15,
        '#maxlength' => 3,
        '#default_value' => $stored_facets[$facet_id]['max_items'],
      );
    }
  }

  $form['filter_settings']['time_filter'] = array(
    '#type' => 'select',
    '#title' => t('Date filter format'),
    '#options' => array(
      'from_to_textfields' => t('Separate textfields for "from"- and "to"-date-values'),
      'range_selection' => t('The user can select a time range (last day, last month, ...)'),
    ),
    '#empty_option' => t('No time filter'),
    '#empty_value' => NULL,
    '#default_value' => $configuration['time_filter'],
  );

  $year = date('Y');
  $form['filter_settings']['time_filter_years'] = array(
    '#type' => 'textfield',
    '#title' => t('Date filter with years'),
    '#description' => t('The year must be less then @year. Leave empty if no years should be displayed in the filter.', array('@year' => $year)),
    '#field_prefix' => t('Add years to the time range filter from this year to') . ' ',
    '#field_suffix' => t('(e.g. 2012)'),
    '#default_value' => $configuration['time_filter_years'],
    '#maxlength' => 4,
    '#attributes' => array('style' => array('width:50px')),
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="time_filter"]' => array('value' => 'range_selection')),
    ),
  );

  if (module_exists('sonr_webmining_trends')) {
    $form['filter_settings']['add_trends'] = array(
      '#type' => 'checkbox',
      '#title' => t('Add Trends chart'),
      '#description' => t('Add a chart showing the trends for the selected filter of the current search.'),
      '#default_value' => $configuration['add_trends'],
    );
  } else {
    $form['filter_settings']['add_trends'] = array(
      '#type' => 'hidden',
      '#default_value' => $configuration['add_trends'],
    );
  }

  $components = array(
    'facets' => 'Facets Filter',
    'time' => 'Date Filter',
    'reset' => 'Reset Button',
  );
  if (module_exists('sonr_webmining_trends')) {
    $components['trends'] = 'Trends area';
  }
  $weighted_components = $configuration['components_order'];
  foreach (array_keys($components) as $component_id) {
    if (!in_array($component_id, $weighted_components)) {
      $weighted_components[] = $component_id;
    }
  }

  foreach ($weighted_components as $weight => $component_id) {
    if (!isset($components[$component_id])) {
      continue;
    }
    $form['filter_settings']['components_order']['#tree'] = TRUE;
    $form['filter_settings']['components_order'][$component_id]['node'] = array(
      '#markup' => $components[$component_id],
    );
    // This field is invisible, but contains sort info (weights).
    $form['filter_settings']['components_order'][$component_id]['weight'] = array(
      '#type' => 'weight',
      '#title_display' => 'invisible',
      '#default_value' => $weight,
    );
  }

  // Tab: Search bar settings.
  $form['searchbar_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search bar settings'),
    '#group' => 'settings',
  );

  $form['searchbar_settings']['show_searchbar'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show search bar'),
    '#default_value' => $configuration['show_searchbar'],
  );

  $form['searchbar_settings']['ac_min_chars'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of characters'),
    '#description' => t('The minimum number of characters you must type before a search is performed (must be greater than 1).'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['ac_min_chars'],
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['searchbar_settings']['ac_max_suggestions'] = array(
    '#type' => 'textfield',
    '#title' => t('Count of suggestions'),
    '#description' => t('The maximum number of items you want to display in the drop down menu.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['ac_max_suggestions'],
    '#element_validate' => array('element_validate_integer_positive'),
  );

  // Tab: Other settings.
  $form['other_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other settings'),
    '#group' => 'settings',
  );

  $form['other_settings']['use_css_file'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use CSS File'),
    '#description' => t('Check if you wish to include the module\'s CSS file.'),
    '#default_value' => $configuration['use_css_file'],
  );

  $form['other_settings']['add_rss_functionality'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add RSS button'),
    '#description' => t('Check if you wish to add the RSS button and its functionallity.'),
    '#default_value' => $configuration['add_rss_functionality'],
  );

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/services/sonr_webmining',
  );

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js');

  return $form;
}

/**
 * Theme function for the configuration set admin form.
 *
 * @param array $variables
 *   An array of variables supported by the theme.
 *
 * @return string
 *   The rendered HTML content of the admin form.
 */
function theme_sonr_webmining_configuration_set_form($variables) {
  $form = $variables['form'];

  // If the facets are not only an empty array (on creation of a new
  // configuration set), create a sortable table out of the facet-list.
  if (!isset($form['filter_settings']['facets_to_show']['#value'])) {
    $rows = array();
    foreach (element_children($form['filter_settings']['facets_to_show']) as $facet_id) {
      $form['filter_settings']['facets_to_show'][$facet_id]['weight']['#attributes']['class'] = array('facets-to-show-order-weight');
      $rows[] = array(
        'data' => array(
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['node']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['selected']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['name']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['aggregated']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['max_items']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['weight']),
        ),
        'class' => array('draggable'),
      );
    }

    $header = array(t('Facet name'), t('Show'), t('Filter name'), t('Aggregat'), t('Maximum items'), t('Weight'));
    $form['filter_settings']['facets_to_show'] = array(
      '#type' => 'item',
      '#title' => t('Facets to show'),
      '#description' => t('Choose the facets you want your users to see as filter-sets and their order.'),
      '#markup' => theme(
        'table',
        array(
          'header' => $header,
          'rows' => $rows,
          'attributes' => array('id' => 'facets-to-show-order'),
        )
      ),
    );

    drupal_add_tabledrag('facets-to-show-order', 'order', 'sibling', 'facets-to-show-order-weight');
  }

  $rows = array();
  foreach (element_children($form['filter_settings']['components_order']) as $component_id) {
    $form['filter_settings']['components_order'][$component_id]['weight']['#attributes']['class'] = array('components-order-order-weight');
    $rows[] = array(
      'data' => array(
        array('class' => array('slide-cross')),
        drupal_render($form['filter_settings']['components_order'][$component_id]['node']),
        drupal_render($form['filter_settings']['components_order'][$component_id]['weight']),
      ),
      'class' => array('draggable'),
    );
  }

  $header = array('Order', t('Component name'), t('Weight'));
  $form['filter_settings']['components_order'] = array(
    '#type' => 'item',
    '#title' => t('Order of the filter components'),
    '#description' => t('Drag and drop the handles of the filter components to set their order.'),
    '#markup' => theme(
      'table',
      array(
        'header' => $header,
        'rows' => $rows,
        'attributes' => array('id' => 'components-order-order'),
      )
    ),
  );

  drupal_add_tabledrag('components-order-order', 'order', 'sibling', 'components-order-order-weight');

  $output = drupal_render_children($form);
  return $output;
}

/**
 * The validation handler for the sOnr webMining configuration admin form.
 */
function sonr_webmining_configuration_set_form_validate($form, &$form_state) {
  if ($form_state['values']['ac_min_chars'] <= 1) {
    form_set_error('ac_min_chars', t('%field must be greater than 1.', array('%field' => t('Number of characters'))));
  }
  $year = date('Y');
  if (!empty($form_state['values']['time_filter_years']) && $form_state['values']['time_filter_years'] >= $year) {
    form_set_error('time_filter_years', t('%field must be less than %year.', array('%field' => t('Date filter with years'), '%year' => $year)));
  }
  if ($form_state['values']['show_similar'] && !$form_state['values']['similar_max_items']) {
    form_set_error('similar_max_items', t('%field can not be empty.', array('%field' => t('Number of documents'))));
  }
}

/**
 * The submit handler for the sOnr webMining configuration admin form.
 */
function sonr_webmining_configuration_set_form_submit($form, &$form_state) {
  $facets_to_show = array();
  $facet_max_items = 10;
  foreach ($form_state['values']['facets_to_show'] as $facet_id => $facet) {
    $weight = $facet['weight'];
    unset($facet['weight']);
    $facet['facet_id'] = $facet_id;
    $facets_to_show[$weight] = $facet;
    if ($facet_max_items < $facet['max_items']) {
      $facet_max_items = $facet['max_items'];
    }
  }
  if (!empty($facets_to_show)) {
    ksort($facets_to_show);
    $facets_to_show = array_values($facets_to_show);
  }

  $components_order = array();
  foreach ($form_state['values']['components_order'] as $component_id => $component) {
    $components_order[$component['weight']] = $component_id;
  }
  if (!empty($components_order)) {
    ksort($components_order);
    $components_order = array_values($components_order);
  }

  $config = array(
    'first_page_only' => $form_state['values']['first_page_only'],
    'items_per_page' => $form_state['values']['items_per_page'],
    'summary_max_chars' => $form_state['values']['summary_max_chars'],
    'separate_blocks' => $form_state['values']['separate_blocks'],
    'add_trends' => $form_state['values']['add_trends'],
    'facets_to_show' => $facets_to_show,
    'time_filter' => $form_state['values']['time_filter'],
    'time_filter_years' => $form_state['values']['time_filter_years'],
    'components_order' => $components_order,
    'show_searchbar' => $form_state['values']['show_searchbar'],
    'ac_max_suggestions' => $form_state['values']['ac_max_suggestions'],
    'ac_min_chars' => $form_state['values']['ac_min_chars'],
    'show_tags' => $form_state['values']['show_tags'],
    'tags_max_items' => $form_state['values']['tags_max_items'],
    'show_similar' => $form_state['values']['show_similar'],
    'similar_max_items' => $form_state['values']['similar_max_items'],
    'show_sentiment' => $form_state['values']['show_sentiment'],
    'use_css_file' => $form_state['values']['use_css_file'],
    'add_rss_functionality' => $form_state['values']['add_rss_functionality'],
    'facet_max_items' => $facet_max_items,
  );

  // Always create a new connection, if URL and type are the same the old one
  // will be used anyway.
  $connection = SemanticConnector::createConnection('sonr_server', $form_state['values']['url'], $form_state['values']['server_title'], array(
    'username' => $form_state['values']['name'],
    'password' => $form_state['values']['pass'],
  ));

  // Update an existing configuration set.
  if (isset($form_state['values']['swid'])) {
    db_update('sonr_webmining_sets')
      ->fields(array(
          'title' => $form_state['values']['title'],
          'sonrid' => $connection->getID(),
          'config' => serialize($config),
        ))
      ->condition('swid', $form_state['values']['swid'])
      ->execute();
  }
  // Create a new configuration set.
  else {
    $swid = db_insert('sonr_webmining_sets')
      ->fields(array(
          'title' => $form_state['values']['title'],
          'sonrid' => $connection->getID(),
        ))
      ->execute();

    // Add all facets once the set is created.
    $aggregator = new SonrWebmining($swid);
    $config['facets_to_show'] = array_keys($aggregator->getAllFacets());
    db_update('sonr_webmining_sets')
      ->fields(array(
          'config' => serialize($config),
        ))
      ->condition('swid', $swid)
      ->execute();

  }
  drupal_set_message(t('sOnr webMining Content aggregator configuration-set %title has been saved.', array('%title' => $form_state['values']['title'])));
  $form_state['redirect'] = 'admin/config/services/sonr_webmining';
}

/**
 * The confirmation-form for deleting a sw configuration set.
 */
function sonr_webmining_configuration_set_delete_form($form, &$form_state, $config) {
  $form_state['config'] = $config;
  return confirm_form($form,
    t('Are you sure you want to delete the sOnr webMining configuration "%title"?', array('%title' => $config->getTitle())),
    'admin/config/services/sonr_webmining',
    t('This action cannot be undone.'),
    t('Delete configuration'));
}

/**
 * The submit handler confirmation-form for deleting a sw configuration set.
 */
function sonr_webmining_configuration_set_delete_form_submit($form, &$form_state) {
  $config = $form_state['config'];

  db_delete('sonr_webmining_sets')
    ->condition('swid', $config->getId())
    ->execute();

  $block_delta_condition = db_or()
    ->condition('delta', 'sonr_webmining_content_' . $config->getId())
    ->condition('delta', 'sonr_webmining_filters_' . $config->getId());

  db_delete('block')
    ->condition($block_delta_condition)
    ->execute();

  drupal_set_message(t('%title has been deleted.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/services/sonr_webmining';
}

/******************************************************
 * Agents functions
 *****************************************************/

/**
 * List all saved sOnr webMining agents.
 *
 * @return string
 *   The rendered HTML of the list of agents.
 */
function sonr_webmining_agent_list() {
  $tables = '';
  $connections = SemanticConnector::getConnectionsByType('sonr_server');
  if (!empty($connections)) {
    $intervals = _sonr_webmining_get_intervals();
    foreach ($connections as $connection) {
      $sonr_api = $connection->getAPI();
      $rows = array();
      $indexed_agents = array();
      if (($agents = $sonr_api->getAgents()) !== FALSE) {
        $indexed_agents = $sonr_api->getIndexedAgents();
        foreach ($agents as $agent) {
          $actions = array(
            l(t('Edit'), 'admin/config/services/sonr_webmining/agents/' . $connection->getID() . '-' . $agent->id . '/edit'),
            l(t('Delete'), 'admin/config/services/sonr_webmining/agents/' . $connection->getID() . '-' . $agent->id . '/delete'),
            l(t('Run manually'), 'admin/config/services/sonr_webmining/agents/' . $connection->getID() . '-' . $agent->id . '/run'),
          );
          $rows[] = array(
            $agent->configuration['source'],
            l($agent->configuration['url'], $agent->configuration['url'], array('attributes' => array('target' => '_blank'))),
            $intervals[$agent->configuration['periodMillis']],
            _sonr_webmining_percentage_bar($agent),
            implode(' | ', $actions),
          );
          if (isset($indexed_agents[$agent->configuration['source']])) {
            unset($indexed_agents[$agent->configuration['source']]);
          }
        }
      }
      $table = array(
        'header' => array(
          t('Source name'),
          t('URL'),
          t('Interval'),
          t('Next run'),
          t('Actions'),
        ),
        'rows' => $rows,
      );
      $class = $tables != '' ? 'class="sw-tablelist"' : '';
      $title = '<h3 ' . $class . '>' . t('Agents for sOnr server "!link"', array('!link' => l($connection->getTitle(), 'admin/config/services/sonr-api/' . $connection->getID()))) . '</h3>';
      $tables .= $title . theme('table', $table);

      if (!empty($indexed_agents)) {
        $rows = array();
        foreach ($indexed_agents as $agent) {
          $actions = array(
            l(t('Delete indexed feed items'), 'admin/config/services/sonr_webmining/agents/' . $connection->getID() . '/' . $agent . '/delete'),
          );
          $rows[] = array(
            $agent,
            implode(' | ', $actions),
          );
        }
        $table = array(
          'header' => array(
            t('Source name'),
            t('Actions'),
          ),
          'rows' => $rows,
        );
        $fieldset = array(
          '#title' => t('Agents that have been deleted, but their feed items are still stored in the search index'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#children' => theme('table', $table),
          '#attributes' => array('class' => array('collapsible', 'collapsed')),
        );
        $tables .= theme('fieldset', array('element' => $fieldset));
        drupal_add_js('misc/collapse.js');
        drupal_add_js('misc/form.js');
      }
    }
  }
  drupal_add_css(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css');

  return $tables;
}

/**
 * The form to add or edit a sOnr webMining agents.
 *
 * @param array $conf
 *   Array of saved configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_agent_form($form, &$form_state, $conf = NULL) {
  $form['source'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the source.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => (!is_null($conf)) ? $conf['source'] : '',
    '#required' => TRUE,
  );

  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('Feed URL of the source.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => (!is_null($conf)) ? $conf['url'] : '',
    '#required' => TRUE,
  );

  /* The space key is not important for now
  $form['spaceKey'] = array(
    '#type' => 'textfield',
    '#title' => t('Type'),
    '#description' => t('The type of the source (e.g. internal, external) shown in a separate facet.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => (!is_null($conf)) ? $conf['spaceKey'] : '',
  );
  */

  $form['periodMillis'] = array(
    '#type' => 'select',
    '#title' => t('Crowling period'),
    '#description' => t('The period when the source is to be re-crawled.'),
    '#options' => _sonr_webmining_get_intervals(),
    '#default_value' => (!is_null($conf)) ? $conf['periodMillis'] : '86400000',
    '#required' => TRUE,
  );

  // Update existing config.
  if (!is_null($conf)) {
    $form['sonrid'] = array(
      '#type' => 'hidden',
      '#value' => $conf['sonrid'],
    );
    $form['agentid'] = array(
      '#type' => 'hidden',
      '#value' => $conf['id'],
    );
  }
  // Create a new config --> choose sOnr-configuration.
  else {
    $sonr_options = array();
    $connections = SemanticConnector::getConnectionsByType('sonr_server');
    if (!empty($connections)) {
      foreach ($connections as $connection) {
        $sonr_options[$connection->getID()] = $connection->getTitle();
      }
    }
    $form['sonrid'] = array(
      '#type' => 'select',
      '#title' => t('sOnr configuration'),
      '#description' => t('Choose the sOnr configuration this agent will be created for.'),
      '#options' => $sonr_options,
      '#required' => TRUE,
    );
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/services/sonr_webmining/agents',
  );

  return $form;
}

/**
 * The validation handler for the sOnr webMining agent admin form.
 */
function sonr_webmining_agent_form_validate($form, &$form_state) {
  if (!valid_url($form_state['values']['url'], TRUE)) {
    form_set_error('url', t('The field URL must be a valid URL.'));
  }
}

/**
 * The submit handler for the sOnr webMining agent admin form.
 */
function sonr_webmining_agent_form_submit($form, &$form_state) {
  global $user;

  $sonr_api = SemanticConnector::getConnection('sonr_server', $form_state['values']['sonrid'])->getAPI();
  $conf = array(
    'privateContent' => 0,
    'username' => $user->name,
    'source' => $form_state['values']['source'],
    'url' => $form_state['values']['url'],
    'periodMillis' => $form_state['values']['periodMillis'],
//    'spaceKey' => $form_state['values']['spaceKey'],
    'spaceKey' => '',
  );

  if (isset($form_state['values']['agentid'])) {
    $response = $sonr_api->updateAgent($form_state['values']['agentid'], $conf);
  }
  else {
    $response = $sonr_api->addAgent($conf);
  }
  if ($response) {
    drupal_set_message(t('sOnr webMining Content Aggregator agent %title has been saved.', array('%title' => $form_state['values']['source'])));
  }
  else {
    drupal_set_message(t('sOnr webMining Content Aggregator agent %title has not been saved (%error).', array('%title' => $form_state['values']['source'], '%error' => $sonr_api->error())), 'error');
  }
  $form_state['redirect'] = 'admin/config/services/sonr_webmining/agents';
}

/**
 * The confirmation-form for deleting a sOnr webMining agent.
 */
function sonr_webmining_agent_delete_form($form, &$form_state, $conf) {
  $form_state['conf'] = $conf;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to delete the agent "%source"?', array('%source' => $conf['source'])) . '<br />' .
    t('This action cannot be undone.') . '</p>',
  );
  $form['deleteIndex'] = array(
    '#type' => 'checkbox',
    '#default_value' => true,
    '#title' => t('Delete all indexed feed items also.'),
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete agent'),
  );

  return $form;
}

/**
 * The submit handler for deleting a sOnr webMining agent.
 */
function sonr_webmining_agent_delete_form_submit($form, &$form_state) {
  $conf = $form_state['conf'];
  $sonr_api = SemanticConnector::getConnection('sonr_server', $conf['sonrid'])->getAPI();
  $response = $sonr_api->deleteAgent($conf['id']);

  if ($form_state['values']['deleteIndex']) {
    $sonr_api->deleteIndex($conf['source']);
  }

  if ($response) {
    drupal_set_message(t('%source has been deleted.', array('%source' => $conf['source'])));
  }
  else {
    drupal_set_message(t('%source has not been deleted (%error).', array('%source' => $conf['source'], '%error' => $sonr_api->error())), 'error');
  }
  $form_state['redirect'] = 'admin/config/services/sonr_webmining/agents';
}

/**
 * The confirmation-form for running a sOnr webMinig agent manually.
 */
function sonr_webmining_agent_run_form($form, &$form_state, $conf) {
  $form_state['conf'] = $conf;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to run the agent "%source"?', array('%source' => $conf['source'])) . '</p>',
  );
  $form['run'] = array(
    '#type' => 'submit',
    '#value' => t('Run agent'),
  );

  return $form;
}

/**
 * The submit handler for running a sOnr webMing agent manually.
 */
function sonr_webmining_agent_run_form_submit($form, &$form_state) {
  $conf = $form_state['conf'];
  $sonr_api = SemanticConnector::getConnection('sonr_server', $conf['sonrid'])->getAPI();
  $response = $sonr_api->runAgent($conf['id']);

  if ($response) {
    drupal_set_message(t('%source has been started.', array('%source' => $conf['source'])));
  }
  else {
    drupal_set_message(t('%source has not been started (%error).', array('%source' => $conf['source'], '%error' => $sonr_api->error())), 'error');
  }
  $form_state['redirect'] = 'admin/config/services/sonr_webmining/agents';
}

/**
 * The confirmation-form for deleting a sOnr webMining agent.
 */
function sonr_webmining_agent_index_delete_form($form, &$form_state, $sonr_id, $agent) {
  $form_state['sonrid'] = $sonr_id;
  $form_state['agent'] = $agent;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to delete all indexed feed items for the agent "%agent"?', array('%agent' => $agent)) . '<br />' .
      t('This action cannot be undone.') . '</p>',
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete indexed feed times'),
  );

  return $form;
}

/**
 * The submit handler for deleting a sOnr webMining agent.
 */
function sonr_webmining_agent_index_delete_form_submit($form, &$form_state) {
  $sonr_api = SemanticConnector::getConnection('sonr_server', $form_state['sonrid'])->getAPI();
  $response = $sonr_api->deleteIndex($form_state['agent']);

  if ($response) {
    drupal_set_message(t('Indexed feed items have been deleted for agent %agent.', array('%agent' => $form_state['agent'])));
  }
  else {
    drupal_set_message(t('Indexed feed items have not been deleted for agent %agent (%error).', array('%agent' => $form_state['agent'], '%error' => $sonr_api->error())), 'error');
  }
  $form_state['redirect'] = 'admin/config/services/sonr_webmining/agents';
}

/**
 * Returns different time intervals.
 *
 * @return array
 *  Array with different time intervals.
 */
function _sonr_webmining_get_intervals() {
  return array(
    '3600000' => t('hourly'),
    '86400000' => t('daily'),
    '604800000' => t('weekly'),
  );
}

/**
 * Shows a bar in percentage when the next run starts.
 * TODO: If an Agent has problems then send an email (adapt sOnr server configuration)
 *
 * @param object $agent
 *   An agent
 *
 * @return string
 *   The bar
 */
function _sonr_webmining_percentage_bar($agent) {
  if ($agent->status['running']) {
    return '<div class="percentage-bar running"><span class="bar" style="width: 100%"></span>
      <span class="tooltip">
        <strong>' . t('Agent is running') . '</strong>
      </span></div>
    ';
  }

  $period = $agent->configuration['periodMillis'] / 1000;
  $time_to_next_run = $agent->status['nextRun'] / 1000 - time();
  $time_to_last_run = time() - $agent->status['lastRun'] / 1000;

  if ($time_to_next_run < 0) {
    $last_run = _sonr_webmining_time_to_text($time_to_last_run);
    return '<div class="percentage-bar error"><span class="bar" style="width: 100%"></span>
      <span class="tooltip">
        <strong>' . t('Last run') . ':</strong><br />' . $last_run . ' ago<br />
        <strong>' . t('Agent problem') . ':</strong><br />' . t('Please start agent manually!') . '
      </span></div>
    ';
  }

  $percentage = round(100 * $time_to_last_run / $period);

  $next_run = _sonr_webmining_time_to_text($time_to_next_run, 'ceil');
  $last_run = _sonr_webmining_time_to_text($time_to_last_run, 'floor');

  return '<div class="percentage-bar"><span class="bar" style="width: ' . $percentage . '%"></span>
    <span class="tooltip">
      <strong>' . t('Next run') . ':</strong><br /> ' . t('in @time', array('@time' => $next_run)) . '<br />
      <strong>' . t('Last run') . ':</strong><br /> ' . t('@time ago', array('@time' => $last_run)) . '
    </span></div>
  ';
}

/**
 * Converts a timestamp into human readable format
 *
 * @param int $time
 *   A timestamp
 * @param string $round
 *   Type of rounding (round|ceil|floor)
 *
 * @return string
 *   A human readable time
 */
function _sonr_webmining_time_to_text($time, $round='round') {
  $time = $time / 60;
  $minutes = (int) $time;
  if ($minutes < 60) {
    return t('!minutes min', array('!minutes' => $minutes));
  }

  $time = $time / 60;
  $hours = (int) $time;
  $minutes = $round(($time - $hours) * 60);
  if ($minutes >= 60) {
    $hours++;
    $minutes = 0;
  }
  if ($hours < 24) {
    return t('!hours hr !minutes min', array('!hours' => $hours, '!minutes' => $minutes));
  }

  $time = $time / 24;
  $days = (int) $time;
  $hours = $round(($time - $days) * 24);
  if ($hours >= 24) {
    $days++;
    $hours = 0;
  }
  return t('!days days !hours hours', array('!days' => $days, '!hours' => $hours));
}
